#ifndef CLOUDCOMM_H
#define CLOUDCOMM_H

#include "common.h"
#include <pthread.h>
/**
 * This class provides a communication API to the webserver.  It also
 * validates the HMACs on the slots and handles encryption.
 * @author Brian Demsky <bdemsky@uci.edu>
 * @version 1.0
 */

#define CloudComm_SALT_SIZE 8
#define CloudComm_TIMEOUT_MILLIS 5000
;	// 100
#define CloudComm_IV_SIZE 16
/** Sets the size for the HMAC. */
#define CloudComm_HMAC_SIZE 32
#define HttpURLConnection_HTTP_OK 200

typedef struct {
	int fd;
	int numBytes;
} WebConnection;


class CloudComm {
private:
	IoTString *baseurl;
	AESKey *key;
	Mac *mac;
	IoTString *password;
	SecureRandom *random;
	Array<char> *salt;
	Table *table;
	int32_t listeningPort;
	pthread_t localServerThread;
	bool doEnd;
	TimingSingleton *timer;
	Array<char> *getslot;
	Array<char> *putslot;

	/**
	 * Generates Key from password.
	 */
	AESKey *initKey();

	/**
	 * Inits the HMAC generator.
	 */
	void initCrypt();

	/*
	 * Builds the URL for the given request.
	 */
	IoTString *buildRequest(bool isput, int64_t sequencenumber, int64_t maxentries);
	void setSalt();
	bool getSalt();
	Array<char> *createIV(int64_t machineId, int64_t localSequenceNumber);
	Array<char> *createIV();
	Array<char> *encryptSlotAndPrependIV(Array<char> *rawData, Array<char> *ivBytes);
	Array<char> *stripIVAndDecryptSlot(Array<char> *rawData);
	Array<Slot *> *processSlots(WebConnection *wc);


public:
	/**
	 * Empty Constructor needed for child class.
	 */
	CloudComm();

	/**
	 * Constructor for actual use. Takes in the url and password.
	 */
	CloudComm(Table *_table,  IoTString *_baseurl, IoTString *_password, int _listeningPort);
	~CloudComm();

	/**
	 * Inits all the security stuff
	 */
	void initSecurity();

	/*
	 * API for putting a slot into the queue.  Returns NULL on success.
	 * On failure, the server will send slots with newer sequence
	 * numbers.
	 */
	Array<Slot *> *putSlot(Slot *slot, int max);

	/**
	 * Request the server to send all slots with the given
	 * sequencenumber or newer.
	 */
	Array<Slot *> *getSlots(int64_t sequencenumber);


	/**
	 * Method that actually handles building Slot objects from the
	 * server response.  Shared by both putSlot and getSlots.
	 */

	Array<char> *sendLocalData(Array<char> *sendData, int64_t localSequenceNumber, IoTString *host, int port);
	void closeCloud();
	void localServerWorkerFunction();
};
#endif
